import koji

import fedmsg.consumers
import robosignatory.utils as utils

import logging
log = logging.getLogger("robosignatory")


class TagSignerConsumer(fedmsg.consumers.FedmsgConsumer):
    topic = 'org.fedoraproject.prod.buildsys.tag'
    config_key = 'robosignatory.enabled.tagsigner'

    def __init__(self, *args, **kwargs):
        super(TagSignerConsumer, self).__init__(*args, **kwargs)
        self.sigul_helper = utils.SigulHelper(
            self.hub.config['robosignatory.signing.user'],
            self.hub.config['robosignatory.signing.passphrase_file'],
            self.hub.config['robosignatory.signing.config_file'])
        self.koji_clients = {}
        for instance in self.hub.config['robosignatory.koji_instances']:
            instance_info = self.hub.config[
                'robosignatory.koji_instances'][instance]
            client = koji.ClientSession(instance_info['url'], {})
            if instance_info['options']['authmethod'] != 'ssl':
                raise Exception('Only SSL authmethod supported')
            client.ssl_login(instance_info['options']['cert'],
                             None,
                             instance_info['options']['serverca'])
            instance_obj = {'client': client,
                            'tags': {}}
            for tag in instance_info['tags']:
                if tag['from'] in instance_obj['tags']:
                    raise Exception('From detected twice: %s' % tag['from'])
                instance_obj['tags'][tag['from']] = {'to': tag['to'],
                                                     'key': tag['key'],
                                                     'keyid': tag['keyid']}
            
            self.koji_clients[instance] = instance_obj

            log.info('TagSignerConsumer ready for service')


    def consume(self, msg):
        msg_id = msg['body'].get('msg_id', None)
        msg = msg['body']['msg']

        #  {u'build_id': 799208,
        #   u'name': u'python-fmn-rules',
        #   u'tag_id': 374,
        #   u'instance': u'primary',
        #   u'tag': u'epel7-infra',
        #   u'user': u'puiterwijk',
        #   u'version': u'0.9.1',
        #   u'owner': u'sayanchowdhury',
        #   u'release': u'1.el7'}}

        build_nvr = '%(name)s-%(version)s-%(release)s' % msg
        build_id = msg['build_id']
        tag = msg['tag']
        koji_instance = msg['instance']

        log.info('Build %s (%s) tagged into %s on %s',
                 build_nvr, build_id, tag, koji_instance)

        if koji_instance not in self.koji_clients:
            log.info('Koji instance not known, skipping')
            return

        instance = self.koji_clients[koji_instance]
        if tag not in instance['tags']:
            log.info('Tag not autosigned, skipping')
            return

        tag_info = instance['tags'][tag]
        log.info('Going to sign with %s (%s) and move to %s',
                 tag_info['key'], tag_info['keyid'], tag_info['to'])

        rpms, rpmids = utils.get_rpms(instance['client'], build_id=build_id)
        log.info('RPMs to sign and move: %s (%s)',
                 ', '.join(rpms), ', '.join(rpmids))

        cmdline = self.sigul_helper.build_sign_cmdline(tag_info['key'],
                                                       rpms,
                                                       koji_instance)
        log.debug('Signing command line: %s' % cmdline)

        ret, stdout, stderr = utils.run_command(cmdline)
        if ret != 0:
            log.error('Error signing! Signing output: %s, stdout: '
                      '%s, stderr: %s', ret, stdout, stderr)
            return
        
        log.info('Packages correctly signed, moving to %s' % tag_info['to'])
        instance['client'].tagBuild(tag_info['to'], build_id, False, tag)
        log.info('Build was succesfully signed and moved, telling koji to '
                 'write with key id %s', tag_info['keyid'])

        for rpmid in rpmids:
            instance['client'].writeSignedRPM(rpmid, tag_info['keyid'])

        log.info('Signed RPMs written out')
